package com.usky.eg.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.eg.domain.EgDevice;
import com.usky.eg.service.EgDeviceService;
import com.usky.eg.service.vo.EgDeviceRequestVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * <p>
 * 门禁_设备信息表 前端控制器
 * </p>
 *
 * @author zyj
 * @since 2024-11-27
 */
@RestController
@RequestMapping("/egDevice")
public class EgDeviceController {
    @Autowired
    private EgDeviceService egDeviceService;

    @PostMapping("page")
    public ApiResult<CommonPage<EgDevice>> page(@RequestBody EgDeviceRequestVO requestVO){
        return ApiResult.success(egDeviceService.page(requestVO));
    }

    /**
     * 新增
     * @param egDevice
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody EgDevice egDevice){
        egDeviceService.add(egDevice);
        return ApiResult.success();
    }

    /**
     * 修改
     * @param egDevice
     * @return
     */
    @PutMapping
    public ApiResult<Void> update(@RequestBody EgDevice egDevice){
        egDeviceService.update(egDevice);
        return ApiResult.success();
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id){
        egDeviceService.remove(id);
        return ApiResult.success();
    }

    /**
     * 下发设备控制命令
     */
    @Log(title = "门禁系统人员校验开门", businessType = BusinessType.OTHER)
    @GetMapping("/control")
    public ApiResult<Map<String,Object>> control(@RequestParam("productCode") String productCode,
                                                 @RequestParam("deviceUuid") String deviceUuid,
                                                 @RequestParam("commandCode") String commandCode,
                                                 @RequestParam("commandValue") String commandValue,
                                                 @RequestParam(value = "domain",required = false) String domain,
                                                 @RequestParam(value = "userId",required = false) Long userId,
                                                 @RequestParam(value = "userName",required = false) String userName){
        return ApiResult.success(egDeviceService.control(productCode,deviceUuid,commandCode,commandValue,domain,userId,userName));
    }

}

