package com.usky.fire.service.util;

import com.ruoyi.common.core.annotation.Excel;
import com.ruoyi.common.core.annotation.Excels;
import com.ruoyi.common.core.annotation.Excel.ColumnType;
import com.ruoyi.common.core.annotation.Excel.Type;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.file.FileTypeUtils;
import com.ruoyi.common.core.utils.file.ImageUtils;
import com.ruoyi.common.core.utils.poi.ExcelHandlerAdapter;
import com.ruoyi.common.core.utils.reflect.ReflectUtils;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.RegExUtils;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.ClientAnchor;
import org.apache.poi.ss.usermodel.DataValidation;
import org.apache.poi.ss.usermodel.DataValidationConstraint;
import org.apache.poi.ss.usermodel.DataValidationHelper;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Drawing;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFDataValidation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @description：TODO
 * @author： fu
 * @create： 2024-04-30 12:34
 */
public class ExcelUtilImage<T> {
    private static final Logger log = LoggerFactory.getLogger(ExcelUtilImage.class);
    public static final String FORMULA_REGEX_STR = "=|-|\\+|@";
    public static final String[] FORMULA_STR = new String[]{"=", "-", "+", "@"};
    public static final int sheetSize = 65536;
    private String sheetName;
    private Type type;
    private Workbook wb;
    private Sheet sheet;
    private Map<String, CellStyle> styles;
    private List<T> list;
    private List<Object[]> fields;
    private int rownum;
    private String title;
    private short maxHeight;
    private Map<Integer, Double> statistics = new HashMap();
    private static final DecimalFormat DOUBLE_FORMAT = new DecimalFormat("######0.00");
    public Class<T> clazz;

    public ExcelUtilImage(Class<T> clazz) {
        this.clazz = clazz;
    }

    public void init(List<T> list, String sheetName, String title, Type type) {
        if (list == null) {
            list = new ArrayList();
        }

        this.list = (List) list;
        this.sheetName = sheetName;
        this.type = type;
        this.title = title;
        this.createExcelField();
        this.createWorkbook();
        this.createTitle();
    }

    public void createTitle() {
        if (StringUtils.isNotEmpty(this.title)) {
            Sheet var10000 = this.sheet;
            int var10001;
            if (this.rownum == 0) {
                int var10003 = this.rownum;
                var10001 = var10003;
                this.rownum = var10003 + 1;
            } else {
                var10001 = 0;
            }

            Row titleRow = var10000.createRow(var10001);
            titleRow.setHeightInPoints(30.0F);
            Cell titleCell = titleRow.createCell(0);
            titleCell.setCellStyle((CellStyle) this.styles.get("title"));
            titleCell.setCellValue(this.title);
            this.sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(), titleRow.getRowNum(), titleRow.getRowNum(), this.fields.size() - 1));
        }

    }

    public List<T> importExcel(InputStream is) throws Exception {
        return this.importExcel(is, 0);
    }

    public List<T> importExcel(InputStream is, int titleNum) throws Exception {
        return this.importExcel("", is, titleNum);
    }

    public List<T> importExcel(String sheetName, InputStream is, int titleNum) throws Exception {
        this.type = Type.IMPORT;
        this.wb = WorkbookFactory.create(is);
        List<T> list = new ArrayList();
        Sheet sheet = StringUtils.isNotEmpty(sheetName) ? this.wb.getSheet(sheetName) : this.wb.getSheetAt(0);
        if (sheet == null) {
            throw new IOException("文件sheet不存在");
        } else {
            int rows = sheet.getLastRowNum();
            if (rows > 0) {
                Map<String, Integer> cellMap = new HashMap();
                Row heard = sheet.getRow(titleNum);

                for (int i = 0; i < heard.getPhysicalNumberOfCells(); ++i) {
                    Cell cell = heard.getCell(i);
                    if (StringUtils.isNotNull(cell)) {
                        String value = this.getCellValue(heard, i).toString();
                        cellMap.put(value, i);
                    } else {
                        cellMap.put(String.valueOf((Object) null), i);
                    }
                }

                List<Object[]> fields = this.getFields();
                Map<Integer, Object[]> fieldsMap = new HashMap();
                Iterator var24 = fields.iterator();

                while (var24.hasNext()) {
                    Object[] objects = (Object[]) var24.next();
                    Excel attr = (Excel) objects[1];
                    Integer column = (Integer) cellMap.get(attr.name());
                    if (column != null) {
                        fieldsMap.put(column, objects);
                    }
                }

                for (int i = titleNum + 1; i <= rows; ++i) {
                    Row row = sheet.getRow(i);
                    if (!this.isRowEmpty(row)) {
                        T entity = null;
                        Iterator var28 = fieldsMap.entrySet().iterator();

                        while (var28.hasNext()) {
                            Entry<Integer, Object[]> entry = (Entry) var28.next();
                            Object val = this.getCellValue(row, (Integer) entry.getKey());
                            entity = entity == null ? this.clazz.newInstance() : entity;
                            Field field = (Field) ((Object[]) entry.getValue())[0];
                            Excel attr = (Excel) ((Object[]) entry.getValue())[1];
                            Class<?> fieldType = field.getType();
                            String propertyName;
                            if (String.class == fieldType) {
                                propertyName = Convert.toStr(val);
                                if (StringUtils.endsWith(propertyName, ".0")) {
                                    val = StringUtils.substringBefore(propertyName, ".0");
                                } else {
                                    String dateFormat = ((Excel) field.getAnnotation(Excel.class)).dateFormat();
                                    if (StringUtils.isNotEmpty(dateFormat)) {
                                        val = this.parseDateToStr(dateFormat, val);
                                    } else {
                                        val = Convert.toStr(val);
                                    }
                                }
                            } else if ((Integer.TYPE == fieldType || Integer.class == fieldType) && StringUtils.isNumeric(Convert.toStr(val))) {
                                val = Convert.toInt(val);
                            } else if ((Long.TYPE == fieldType || Long.class == fieldType) && StringUtils.isNumeric(Convert.toStr(val))) {
                                val = Convert.toLong(val);
                            } else if (Double.TYPE != fieldType && Double.class != fieldType) {
                                if (Float.TYPE != fieldType && Float.class != fieldType) {
                                    if (BigDecimal.class == fieldType) {
                                        val = Convert.toBigDecimal(val);
                                    } else if (Date.class == fieldType) {
                                        if (val instanceof String) {
                                            val = DateUtils.parseDate(val);
                                        } else if (val instanceof Double) {
                                            val = DateUtil.getJavaDate((Double) val);
                                        }
                                    } else if (Boolean.TYPE == fieldType || Boolean.class == fieldType) {
                                        val = Convert.toBool(val, false);
                                    }
                                } else {
                                    val = Convert.toFloat(val);
                                }
                            } else {
                                val = Convert.toDouble(val);
                            }

                            if (StringUtils.isNotNull(fieldType)) {
                                propertyName = field.getName();
                                if (StringUtils.isNotEmpty(attr.targetAttr())) {
                                    propertyName = field.getName() + "." + attr.targetAttr();
                                } else if (StringUtils.isNotEmpty(attr.readConverterExp())) {
                                    val = reverseByExp(Convert.toStr(val), attr.readConverterExp(), attr.separator());
                                } else if (!attr.handler().equals(ExcelHandlerAdapter.class)) {
                                    val = this.dataFormatHandlerAdapter(val, attr);
                                }

                                ReflectUtils.invokeSetter(entity, propertyName, val);
                            }
                        }

                        list.add(entity);
                    }
                }
            }

            return list;
        }
    }

    public void exportExcel(HttpServletResponse response, List<T> list, String sheetName) {
        this.exportExcel(response, list, sheetName, "");
    }

    public void exportExcel(HttpServletResponse response, List<T> list, String sheetName, String title) {
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        this.init(list, sheetName, title, Type.EXPORT);
        this.exportExcel(response);
    }

    public void importTemplateExcel(HttpServletResponse response, String sheetName) {
        this.importTemplateExcel(response, sheetName, "");
    }

    public void importTemplateExcel(HttpServletResponse response, String sheetName, String title) {
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        this.init((List) null, sheetName, title, Type.IMPORT);
        this.exportExcel(response);
    }

    public void exportExcel(HttpServletResponse response) {
        try {
            this.writeSheet();
            this.addSignatureAndDate(this.sheet); // 写入数据后添加签名和日期
            this.wb.write(response.getOutputStream());
        } catch (Exception var6) {
            log.error("导出Excel异常{}", var6.getMessage());
        } finally {
            IOUtils.closeQuietly(this.wb);
        }

    }

    public void addSignatureAndDate(Sheet sheet) {
        try {
            // 获取最后一行的行号
            int lastRowNum = sheet.getLastRowNum();
            // 检查是否需要创建新行，如果最后一行不为空，则在其下创建新行
            int newRowNum = (lastRowNum == 0 || !isRowEmpty(sheet.getRow(lastRowNum))) ? lastRowNum + 2 : lastRowNum;
            Row signatureAndDateRow = sheet.createRow(newRowNum);
            Cell signatureCell = signatureAndDateRow.createCell(0);
            signatureCell.setCellValue("负责人签名：");
            Cell dateCell = signatureAndDateRow.createCell(4);
            dateCell.setCellValue("日期：");
            CellStyle style = this.wb.createCellStyle();
            Font font = this.wb.createFont();
            font.setBold(true);
            style.setFont(font);
            signatureCell.setCellStyle(style);
            dateCell.setCellStyle(style);

        } catch (Exception e) {
            log.error("Error adding signature and date to the Excel sheet.", e);
        }
    }

    public void writeSheet() {
        int sheetNo = Math.max(1, (int) Math.ceil((double) this.list.size() * 1.0D / 65536.0D));

        for (int index = 0; index < sheetNo; ++index) {
            this.createSheet(sheetNo, index);
            Row row = this.sheet.createRow(this.rownum);
            int column = 0;
            Iterator var5 = this.fields.iterator();

            while (var5.hasNext()) {
                Object[] os = (Object[]) var5.next();
                Excel excel = (Excel) os[1];
                this.createCell(excel, row, column++);
            }

            if (Type.EXPORT.equals(this.type)) {
                this.fillExcelData(index, row);
                this.addStatisticsRow();
            }
        }

    }


    public void fillExcelData(int index, Row row) {
        int startNo = index * 65536;
        int endNo = Math.min(startNo + 65536, this.list.size());

        for (int i = startNo; i < endNo; ++i) {
            row = this.sheet.createRow(i + 1 + this.rownum - startNo);
            T vo = this.list.get(i);
            int column = 0;
            Iterator var8 = this.fields.iterator();

            while (var8.hasNext()) {
                Object[] os = (Object[]) var8.next();
                Field field = (Field) os[0];
                Excel excel = (Excel) os[1];
                this.addCell(excel, row, vo, field, column++);
            }
        }
    }

    private Map<String, CellStyle> createStyles(Workbook wb) {
        Map<String, CellStyle> styles = new HashMap();
        CellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        Font titleFont = wb.createFont();
        titleFont.setFontName("Arial");
        titleFont.setFontHeightInPoints((short) 16);
        titleFont.setBold(true);
        style.setFont(titleFont);
        styles.put("title", style);
        style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setBorderRight(BorderStyle.THIN);
        style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderLeft(BorderStyle.THIN);
        style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderTop(BorderStyle.THIN);
        style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setBorderBottom(BorderStyle.THIN);
        style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        Font dataFont = wb.createFont();
        dataFont.setFontName("Arial");
        dataFont.setFontHeightInPoints((short) 10);
        style.setFont(dataFont);
        styles.put("data", style);
        style = wb.createCellStyle();
        style.cloneStyleFrom((CellStyle) styles.get("data"));
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        Font headerFont = wb.createFont();
        headerFont.setFontName("Arial");
        headerFont.setFontHeightInPoints((short) 10);
        headerFont.setBold(true);
        headerFont.setColor(IndexedColors.WHITE.getIndex());
        style.setFont(headerFont);
        styles.put("header", style);
        style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        Font totalFont = wb.createFont();
        totalFont.setFontName("Arial");
        totalFont.setFontHeightInPoints((short) 10);
        style.setFont(totalFont);
        styles.put("total", style);
        style = wb.createCellStyle();
        style.cloneStyleFrom((CellStyle) styles.get("data"));
        style.setAlignment(HorizontalAlignment.LEFT);
        styles.put("data1", style);
        style = wb.createCellStyle();
        style.cloneStyleFrom((CellStyle) styles.get("data"));
        style.setAlignment(HorizontalAlignment.CENTER);
        styles.put("data2", style);
        style = wb.createCellStyle();
        style.cloneStyleFrom((CellStyle) styles.get("data"));
        style.setAlignment(HorizontalAlignment.RIGHT);
        styles.put("data3", style);
        return styles;
    }

    public Cell createCell(Excel attr, Row row, int column) {
        Cell cell = row.createCell(column);
        cell.setCellValue(attr.name());
        this.setDataValidation(attr, row, column);
        cell.setCellStyle((CellStyle) this.styles.get("header"));
        return cell;
    }

    public void setCellVo(Object value, Excel attr, Cell cell) {
        if (ColumnType.STRING == attr.cellType()) {
            String cellValue = Convert.toStr(value);
            if (StringUtils.startsWithAny(cellValue, FORMULA_STR)) {
                cellValue = RegExUtils.replaceFirst(cellValue, "=|-|\\+|@", "\t$0");
            }

            cell.setCellValue(StringUtils.isNull(cellValue) ? attr.defaultValue() : cellValue + attr.suffix());
        } else if (ColumnType.NUMERIC == attr.cellType()) {
            if (StringUtils.isNotNull(value)) {
                cell.setCellValue(StringUtils.contains(Convert.toStr(value), ".") ? Convert.toDouble(value) : (double) Convert.toInt(value));
            }
        } else if (ColumnType.IMAGE == attr.cellType()) {
            ClientAnchor anchor = new XSSFClientAnchor(0, 0, 0, 0, (short) cell.getColumnIndex(), cell.getRow().getRowNum(), (short) (cell.getColumnIndex() + 1), cell.getRow().getRowNum() + 1);
            String imagePath = Convert.toStr(value);
            if (StringUtils.isNotEmpty(imagePath)) {
                byte[] data = ImageUtils.getImage(imagePath);
                getDrawingPatriarch(cell.getSheet()).createPicture(anchor, cell.getSheet().getWorkbook().addPicture(data, this.getImageType(data)));
            }
        }

    }

    public static Drawing<?> getDrawingPatriarch(Sheet sheet) {
        if (sheet.getDrawingPatriarch() == null) {
            sheet.createDrawingPatriarch();
        }

        return sheet.getDrawingPatriarch();
    }

    public int getImageType(byte[] value) {
        String type = FileTypeUtils.getFileExtendName(value);
        if ("JPG".equalsIgnoreCase(type)) {
            return 5;
        } else {
            return "PNG".equalsIgnoreCase(type) ? 6 : 5;
        }
    }

    public void setDataValidation(Excel attr, Row row, int column) {
        if (attr.name().indexOf("注：") >= 0) {
            this.sheet.setColumnWidth(column, 6000);
        } else {
            this.sheet.setColumnWidth(column, (int) ((attr.width() + 0.72D) * 256.0D));
        }

        if (StringUtils.isNotEmpty(attr.prompt()) || attr.combo().length > 0) {
            this.setPromptOrValidation(this.sheet, attr.combo(), attr.prompt(), 1, 100, column, column);
        }

    }

    

    public Cell addCell(Excel attr, Row row, T vo, Field field, int column) {
        Cell cell = null;

        try {
            row.setHeight(this.maxHeight);
            if (attr.isExport()) {
                cell = row.createCell(column);
                int align = attr.align().value();
                cell.setCellStyle((CellStyle)this.styles.get("data" + (align >= 1 && align <= 3 ? align : "")));
                Object value = this.getTargetValue(vo, field, attr);
                if(value instanceof List) {
                    List listValue = (List) value;
                    for(int i = 0; i < listValue.size(); i++) {
                        Cell newCell = row.createCell(column + i);
                        newCell.setCellStyle((CellStyle)this.styles.get("data" + (align >= 1 && align <= 3 ? align : "")));
                        Object elementValue = listValue.get(i);
                        newCell.setCellValue(elementValue.toString());
                    }
                }
                String dateFormat = attr.dateFormat();
                String readConverterExp = attr.readConverterExp();
                String separator = attr.separator();
                if (StringUtils.isNotEmpty(dateFormat) && StringUtils.isNotNull(value)) {
                    cell.setCellValue(this.parseDateToStr(dateFormat, value));
                } else if (StringUtils.isNotEmpty(readConverterExp) && StringUtils.isNotNull(value)) {
                    cell.setCellValue(convertByExp(Convert.toStr(value), readConverterExp, separator));
                } else if (value instanceof BigDecimal && -1 != attr.scale()) {
                    cell.setCellValue(((BigDecimal)value).setScale(attr.scale(), attr.roundingMode()).toString());
                } else if (!attr.handler().equals(ExcelHandlerAdapter.class)) {
                    cell.setCellValue(this.dataFormatHandlerAdapter(value, attr));
                } else {
                    this.setCellVo(value, attr, cell);
                }

                this.addStatisticsData(column, Convert.toStr(value), attr);
            }
        } catch (Exception var12) {
            log.error("导出Excel失败{}", var12);
        }

        return cell;
    }

/*    public Cell addCell(Excel attr, Row row, T vo, Field field, int column) {
        Cell cell = null;

        try {
            row.setHeight(this.maxHeight);
            if (attr.isExport()) {
                cell = row.createCell(column);
                int align = attr.align().value();
                cell.setCellStyle((CellStyle)this.styles.get("data" + (align >= 1 && align <= 3 ? align : "")));
                Object value = this.getTargetValue(vo, field, attr);
                String dateFormat = attr.dateFormat();
                String readConverterExp = attr.readConverterExp();
                String separator = attr.separator();
                if (StringUtils.isNotEmpty(dateFormat) && StringUtils.isNotNull(value)) {
                    cell.setCellValue(this.parseDateToStr(dateFormat, value));
                } else if (StringUtils.isNotEmpty(readConverterExp) && StringUtils.isNotNull(value)) {
                    cell.setCellValue(convertByExp(Convert.toStr(value), readConverterExp, separator));
                } else if (value instanceof BigDecimal && -1 != attr.scale()) {
                    cell.setCellValue(((BigDecimal)value).setScale(attr.scale(), attr.roundingMode()).toString());
                } else if (!attr.handler().equals(ExcelHandlerAdapter.class)) {
                    cell.setCellValue(this.dataFormatHandlerAdapter(value, attr));
                } else {
                    this.setCellVo(value, attr, cell);
                }

                this.addStatisticsData(column, Convert.toStr(value), attr);
            }
        } catch (Exception var12) {
            log.error("导出Excel失败{}", var12);
        }

        return cell;
    }*/

    public void setPromptOrValidation(Sheet sheet, String[] textlist, String promptContent, int firstRow, int endRow, int firstCol, int endCol) {
        DataValidationHelper helper = sheet.getDataValidationHelper();
        DataValidationConstraint constraint = textlist.length > 0 ? helper.createExplicitListConstraint(textlist) : helper.createCustomConstraint("DD1");
        CellRangeAddressList regions = new CellRangeAddressList(firstRow, endRow, firstCol, endCol);
        DataValidation dataValidation = helper.createValidation(constraint, regions);
        if (StringUtils.isNotEmpty(promptContent)) {
            dataValidation.createPromptBox("", promptContent);
            dataValidation.setShowPromptBox(true);
        }

        if (dataValidation instanceof XSSFDataValidation) {
            dataValidation.setSuppressDropDownArrow(true);
            dataValidation.setShowErrorBox(true);
        } else {
            dataValidation.setSuppressDropDownArrow(false);
        }

        sheet.addValidationData(dataValidation);
    }

    public static String convertByExp(String propertyValue, String converterExp, String separator) {
        StringBuilder propertyString = new StringBuilder();
        String[] convertSource = converterExp.split(",");
        String[] var5 = convertSource;
        int var6 = convertSource.length;

        for (int var7 = 0; var7 < var6; ++var7) {
            String item = var5[var7];
            String[] itemArray = item.split("=");
            if (StringUtils.containsAny(separator, propertyValue)) {
                String[] var10 = propertyValue.split(separator);
                int var11 = var10.length;

                for (int var12 = 0; var12 < var11; ++var12) {
                    String value = var10[var12];
                    if (itemArray[0].equals(value)) {
                        propertyString.append(itemArray[1] + separator);
                        break;
                    }
                }
            } else if (itemArray[0].equals(propertyValue)) {
                return itemArray[1];
            }
        }

        return StringUtils.stripEnd(propertyString.toString(), separator);
    }

    public static String reverseByExp(String propertyValue, String converterExp, String separator) {
        StringBuilder propertyString = new StringBuilder();
        String[] convertSource = converterExp.split(",");
        String[] var5 = convertSource;
        int var6 = convertSource.length;

        for (int var7 = 0; var7 < var6; ++var7) {
            String item = var5[var7];
            String[] itemArray = item.split("=");
            if (StringUtils.containsAny(separator, propertyValue)) {
                String[] var10 = propertyValue.split(separator);
                int var11 = var10.length;

                for (int var12 = 0; var12 < var11; ++var12) {
                    String value = var10[var12];
                    if (itemArray[1].equals(value)) {
                        propertyString.append(itemArray[0] + separator);
                        break;
                    }
                }
            } else if (itemArray[1].equals(propertyValue)) {
                return itemArray[0];
            }
        }

        return StringUtils.stripEnd(propertyString.toString(), separator);
    }

    public String dataFormatHandlerAdapter(Object value, Excel excel) {
        try {
            Object instance = excel.handler().newInstance();
            Method formatMethod = excel.handler().getMethod("format", Object.class, String[].class);
            value = formatMethod.invoke(instance, value, excel.args());
        } catch (Exception var5) {
            log.error("不能格式化数据 " + excel.handler(), var5.getMessage());
        }

        return Convert.toStr(value);
    }

    private void addStatisticsData(Integer index, String text, Excel entity) {
        if (entity != null && entity.isStatistics()) {
            Double temp = 0.0D;
            if (!this.statistics.containsKey(index)) {
                this.statistics.put(index, temp);
            }

            try {
                temp = Double.valueOf(text);
            } catch (NumberFormatException var6) {
            }

            this.statistics.put(index, (Double) this.statistics.get(index) + temp);
        }

    }

    public void addStatisticsRow() {
        if (this.statistics.size() > 0) {
            Row row = this.sheet.createRow(this.sheet.getLastRowNum() + 1);
            Set<Integer> keys = this.statistics.keySet();
            Cell cell = row.createCell(0);
            cell.setCellStyle((CellStyle) this.styles.get("total"));
            cell.setCellValue("合计");
            Iterator var4 = keys.iterator();
            while (var4.hasNext()) {
                Integer key = (Integer) var4.next();
                cell = row.createCell(key);
                cell.setCellStyle((CellStyle) this.styles.get("total"));
                cell.setCellValue(DOUBLE_FORMAT.format(this.statistics.get(key)));
            }

            this.statistics.clear();
        }

    }

    private Object getTargetValue(T vo, Field field, Excel excel) throws Exception {
        Object o = field.get(vo);
        if (StringUtils.isNotEmpty(excel.targetAttr())) {
            String target = excel.targetAttr();
            if (target.contains(".")) {
                String[] targets = target.split("[.]");
                String[] var7 = targets;
                int var8 = targets.length;

                for (int var9 = 0; var9 < var8; ++var9) {
                    String name = var7[var9];
                    o = this.getValue(o, name);
                }
            } else {
                o = this.getValue(o, target);
            }
        }

        return o;
    }

    private Object getValue(Object o, String name) throws Exception {
        if (StringUtils.isNotNull(o) && StringUtils.isNotEmpty(name)) {
            Class<?> clazz = o.getClass();
            Field field = clazz.getDeclaredField(name);
            field.setAccessible(true);
            o = field.get(o);
        }

        return o;
    }

    private void createExcelField() {
        this.fields = this.getFields();
        this.fields = (List) this.fields.stream().sorted(Comparator.comparing((objects) -> {
            return ((Excel) objects[1]).sort();
        })).collect(Collectors.toList());
        this.maxHeight = this.getRowHeight();
    }

    public List<Object[]> getFields() {
        List<Object[]> fields = new ArrayList();
        List<Field> tempFields = new ArrayList();
        tempFields.addAll(Arrays.asList(this.clazz.getSuperclass().getDeclaredFields()));
        tempFields.addAll(Arrays.asList(this.clazz.getDeclaredFields()));
        Iterator var3 = tempFields.iterator();

        while (true) {
            Field field;
            do {
                if (!var3.hasNext()) {
                    return fields;
                }

                field = (Field) var3.next();
                if (field.isAnnotationPresent(Excel.class)) {
                    Excel attr = (Excel) field.getAnnotation(Excel.class);
                    if (attr != null && (attr.type() == Type.ALL || attr.type() == this.type)) {
                        field.setAccessible(true);
                        fields.add(new Object[]{field, attr});
                    }
                }
            } while (!field.isAnnotationPresent(Excels.class));

            Excels attrs = (Excels) field.getAnnotation(Excels.class);
            Excel[] excels = attrs.value();
            Excel[] var7 = excels;
            int var8 = excels.length;

            for (int var9 = 0; var9 < var8; ++var9) {
                Excel attr = var7[var9];
                if (attr != null && (attr.type() == Type.ALL || attr.type() == this.type)) {
                    field.setAccessible(true);
                    fields.add(new Object[]{field, attr});
                }
            }
        }
    }

    public short getRowHeight() {
        double maxHeight = 0.0D;

        Excel excel;
        for (Iterator var3 = this.fields.iterator(); var3.hasNext(); maxHeight = Math.max(maxHeight, excel.height())) {
            Object[] os = (Object[]) var3.next();
            excel = (Excel) os[1];
        }

        return (short) ((int) (maxHeight * 20.0D));
    }

    public void createWorkbook() {
        this.wb = new SXSSFWorkbook(500);
        this.sheet = this.wb.createSheet();
        this.wb.setSheetName(0, this.sheetName);
        this.styles = this.createStyles(this.wb);
    }

    public void createSheet(int sheetNo, int index) {
        if (sheetNo > 1 && index > 0) {
            this.sheet = this.wb.createSheet();
            this.createTitle();
            this.wb.setSheetName(index, this.sheetName + index);
        }

    }

    public Object getCellValue(Row row, int column) {
        if (row == null) {
            return row;
        } else {
            Object val = "";

            try {
                Cell cell = row.getCell(column);
                if (StringUtils.isNotNull(cell)) {
                    if (cell.getCellType() != CellType.NUMERIC && cell.getCellType() != CellType.FORMULA) {
                        if (cell.getCellType() == CellType.STRING) {
                            val = cell.getStringCellValue();
                        } else if (cell.getCellType() == CellType.BOOLEAN) {
                            val = cell.getBooleanCellValue();
                        } else if (cell.getCellType() == CellType.ERROR) {
                            val = cell.getErrorCellValue();
                        }
                    } else {
                        val = cell.getNumericCellValue();
                        if (DateUtil.isCellDateFormatted(cell)) {
                            val = DateUtil.getJavaDate((Double) val);
                        } else if ((Double) val % 1.0D != 0.0D) {
                            val = new BigDecimal(val.toString());
                        } else {
                            val = (new DecimalFormat("0")).format(val);
                        }
                    }
                }

                return val;
            } catch (Exception var5) {
                return val;
            }
        }
    }

    private boolean isRowEmpty(Row row) {
        if (row == null) {
            return true;
        } else {
            for (int i = row.getFirstCellNum(); i < row.getLastCellNum(); ++i) {
                Cell cell = row.getCell(i);
                if (cell != null && cell.getCellType() != CellType.BLANK) {
                    return false;
                }
            }

            return true;
        }
    }

    public String parseDateToStr(String dateFormat, Object val) {
        if (val == null) {
            return "";
        } else {
            String str;
            if (val instanceof Date) {
                str = DateUtils.parseDateToStr(dateFormat, (Date) val);
            } else if (val instanceof LocalDateTime) {
                str = DateUtils.parseDateToStr(dateFormat, DateUtils.toDate((LocalDateTime) val));
            } else if (val instanceof LocalDate) {
                str = DateUtils.parseDateToStr(dateFormat, DateUtils.toDate((LocalDate) val));
            } else {
                str = val.toString();
            }

            return str;
        }
    }
}

