package com.usky.iot.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.usky.iot.service.vo.PmReportReadersVO;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 工作报告表
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class PmWorkReport implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 工作报告表主键ID
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 提交人ID
     */
    private Long submitterId;

    /**
     * 报告时间
     */
    private LocalDate reportDate;

    /**
     * 提交时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime submitDate;

    /**
     * 总工时
     */
    private BigDecimal totalHours;

    /**
     * 抄送人
     */
    private String ccTo;

    /**
     * 工作协调
     */
    private String coordinateWork;

    /**
     * 明日计划
     */
    private String tomorrowPlan;

    /**
     * 创建者
     */
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    /**
     * 更新者
     */
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateTime;

    /**
     * 部门ID
     */
    private Long deptId;

    /**
     * 租户ID
     */
    private Integer tenantId;

    /**
     * 工作报告对应内容
     */
    @TableField(exist = false)
    private List<PmWorkContent> workContents;

    /**
     * 已/未读标识
     */
    @TableField(exist = false)
    private Integer readFlag;

    /**
     * 是否发送钉钉 (默认：0，否；1：是)
     */
    private Integer sendDingTalk;

    /**
     * 定时时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timingTime;

    /**
     * 报告状态（默认1,0：保存状态；1：发送状态）
     */
    private Integer reportStatus;

    /**
     * 是否定时（默认0：否，1：是）
     */
    private Integer isRegularlySend;

    /**
     * 已读，未读人员
     */
    @TableField(exist = false)
    private PmReportReadersVO pmReportReaders;

    /**
     * 头像
     */
    @TableField(exist = false)
    private String avatar;

    /**
     * 报告附件
     */
    private String reportFile;

    /**
     * 报告图片
     */
    private String reportImage;

}
