package com.usky.meeting.domain;

import java.time.LocalDateTime;
import java.io.Serializable;
import java.util.List;
import java.util.Set;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.usky.meeting.service.dto.DmMeetingRoomDto;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;

import javax.persistence.*;
import javax.xml.bind.annotation.XmlType;

/**
 * <p>
 * 会议表(预约成功)
 * </p>
 *
 * @author zyj
 * @since 2024-03-08
 */
@Entity
@Data
@EqualsAndHashCode(callSuper = false)
public class MeetingInfo implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @TableId
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private Long meetingId;

    /**
     * 会议议题
     */
    private String meetingName;

    /**
     * 会议室房间
     */
    @Transient
    @TableField(exist = false)
    private MeetingRoom roomInfo;

    /**
     * 会议室id
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    private Long roomId;

    /**
     * 会议室名称
     */
    private String roomName;

    /**
     * 会议介绍
     */
    private String meetingDescribe;

    /**
     * 会议类型(如研讨会、部门会议等)
     */
    private String meetingType;

    /**
     * 参与部门
     */
    private String deptId;

    /**
     * 发起人id
     */
    private String initiatorId;

    /**
     * 发起人信息
     */
    @Transient
    @TableField(exist = false)
    private SysUser initiatorUser;

    /**
     * 参与人信息
     */
    @Transient
    @TableField(exist = false)
    private List<SysUser> users;

    /**
     * 开始时间
     */
    private LocalDateTime startDate;

    /**
     * 结束时间
     */
    private LocalDateTime endDate;

    /**
     * 审批状态(0-审批中 1-审批通过 2-审批驳回)
     */
    private Integer approveStatus;

    /**
     * 审批者意见
     */
    private String approveSuggest;

    /**
     * 实际签到人数
     */
    private Integer actualCount;

    /**
     * 应到人数/计划人数
     */
    private Integer planCount;

    /**
     * 是否延期(会议是否需要延期30分钟)(0-否 1-是)
     */
    private Integer isDelay;

    /**
     * 备注
     */
    private String remark;

    /**
     * 会议状态(0-未开始 1-进行中 2-已结束)
     */
    private Integer meetingStatus;

    /**
     * 创建者
     */
    private String createBy;

    /**
     * 更新者
     */
    private String updateBy;

    /**
     * 创建日期
     */
    private LocalDateTime createTime;

    /**
     * 更新时间
     */
    private LocalDateTime updateTime;

    /**
     * activiti任务id
     */
    private String taskId;

    /**
     * 流程实例id
     */
    private String procInstId;

    /**
     * 是否取消(0-否 1-是)
     */
    private Integer isCancel;

    /**
     * 通知类型(APP/Email/MSG短信)
     */
    private String sendType;

    /**
     * 会议性质(1-高层会议/0-普通会议)
     */
    private Integer meetingProp;

    /**
     * 是否需要审批(0-否 1-是)
     */
    private Integer isApprove;

    /**
     * 高层会议-指定用户
     */
    private String gcAssignUser;

    /**
     * 会议模式(0-线下会议 1-线上会议 2-线下同步线上会议)
     */
    private Integer meetingMode;

    /**
     * 租户ID
     */
    private Integer tenantId;


}
