package com.usky.oa.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import java.time.LocalDateTime;
import java.io.Serializable;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * <p>
 * 请假单据表
 * </p>
 *
 * @author fu
 * @since 2024-12-10
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class OaQjdDocument implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 请假单据表主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 表单id
     */
    private Integer formId;

    /**
     * 1：事假，2：病假（后续附上病假单、挂号单、病历册），3：年假（连续在我司工作一年以上的，才可享受），4：调休（有已获批准的加班流程），
     * 5：婚假（附上结婚证），6：产假（后续附上出生证明），7：陪产假（后续附上出生证明），8：丧假（父母、配偶、子女死亡可申请3天），9：工伤假，
     * 10：孕检假（后续附上挂号单、病例册）
     */
    private String type;

    /**
     * 单据编号
     */
    private String docNo;

    /**
     * 申请人id
     */
    private Long proposer;

    /**
     * 单据状态 (0:未提交可以修改,1:审批中,2:审批通过,3:审批不通过,4:撤销)
     */
    private Integer docStatus;

    /**
     * 开始日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime startTime;

    /**
     * 结束日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime endTime;

    /**
     * 请假时长(小时)
     */
    private Integer duration;

    /**
     * 请假事由
     */
    private String reason;

    /**
     * 图片URL
     */
    private String image;

    /**
     * 创建者
     */
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    /**
     * 更新者
     */
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateTime;

    /**
     * 部门id
     */
    private Long deptId;

    /**
     * 租户id
     */
    private Integer tenantId;


}
