package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaApproval;
import com.usky.oa.domain.OaDocument;
import com.usky.oa.mapper.OaApprovalMapper;
import com.usky.oa.service.OaApprovalService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.vo.OaApprovalCountVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 审批表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-12-10
 */
@Service
public class OaApprovalServiceImpl extends AbstractCrudService<OaApprovalMapper, OaApproval> implements OaApprovalService {

    @Autowired
    private OaApprovalMapper oaApprovalMapper;

    @Override
    public OaApprovalCountVO approvalCount() {
        Long userId = SecurityUtils.getUserId();
        String username = SecurityUtils.getUsername();
        Integer tenantId = SecurityUtils.getTenantId();

        OaApprovalCountVO oaApprovalCountVO = new OaApprovalCountVO();
        // 待处理
        oaApprovalCountVO.setPendingSum(getPendingSum(tenantId, userId).size());
        // 已处理
        oaApprovalCountVO.setAlreadySum(getAlreadySum(tenantId, userId).size());
        // 我发起
        oaApprovalCountVO.setMyInitiated(getMyInitiated(tenantId, username).size());
        // 我收到
        oaApprovalCountVO.setMyReceived(getMyReceived(tenantId, userId).size());

        return oaApprovalCountVO;
    }

    private List<OaApproval> getPendingSum(Integer tenantId, Long userId){
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId)
                .eq(OaApproval::getApprovalStatus, 1);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

    private List<OaApproval> getAlreadySum(Integer tenantId, Long userId){
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId)
                .and(wrapper -> wrapper
                        .eq(OaApproval::getApprovalStatus, 2)
                        .or()
                        .eq(OaApproval::getApprovalStatus, 3));
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

    private List<OaApproval> getMyInitiated(Integer tenantId, String username){
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getCreateBy, username);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals == null ? Collections.emptyList() : oaApprovals;
    }

    private List<OaApproval> getMyReceived(Integer tenantId, Long userId){
        LambdaQueryWrapper<OaApproval> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(OaApproval::getTenantId, tenantId)
                .eq(OaApproval::getApprovalUid, userId);
        List<OaApproval> oaApprovals = oaApprovalMapper.selectList(queryWrapper);
        return oaApprovals== null ? Collections.emptyList() : oaApprovals;
    }

}
