package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaDocument;
import com.usky.oa.domain.OaJbdDocument;
import com.usky.oa.domain.OaQjdDocument;
import com.usky.oa.mapper.OaDocumentMapper;
import com.usky.oa.mapper.OaJbdDocumentMapper;
import com.usky.oa.mapper.OaQjdDocumentMapper;
import com.usky.oa.service.OaDocumentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.vo.OaApprovalCountVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 单据总表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-12-26
 */
@Service
public class OaDocumentServiceImpl extends AbstractCrudService<OaDocumentMapper, OaDocument> implements OaDocumentService {

    @Autowired
    private OaDocumentMapper oaDocumentMapper;

    @Autowired
    private OaQjdDocumentMapper oaQjdDocumentMapper;

    @Autowired
    private OaJbdDocumentMapper oaJbdDocumentMapper;

    // 我的申请
    @Override
    public CommonPage<OaDocument> myApplication(String docNo, String type, Integer status, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();

        LambdaQueryWrapper<OaDocument> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(OaDocument::getTenantId, tenantId).eq(OaDocument::getProposer, userId)
                .like(StringUtils.isNotBlank(docNo), OaDocument::getDocNo, docNo)
                .eq(StringUtils.isNotBlank(type), OaDocument::getType, type)
                .eq(status != null, OaDocument::getDocStatus, status)
                .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaDocument::getCreateTime, startTime, endTime)
                .orderByDesc(OaDocument::getId);

        return ToCommonPage(page(new Page<>(pageNum, pageSize), wrapper));
    }

    @Override
    public Object documentDetails(String docNo) {
        Object documentDetails = null;
        String type = getFormSign(docNo);

        switch (type) {
            case "QJD":
                documentDetails = oaQjdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                documentDetails = oaJbdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }

        return documentDetails;
    }

    @Override
    public void delDocument(String docNo) {
        String type = getFormSign(docNo);
        switch (type) {
            case "QJD":
                oaQjdDocumentMapper.delete(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.delete(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }
        oaDocumentMapper.delete(Wrappers.lambdaQuery(OaDocument.class).eq(OaDocument::getDocNo, docNo));
    }

    @Override
    public void addDoc(Object oaDocument) {

    }

    @Override
    public OaDocument updateDoc(OaDocument oaDocument) {
        return null;
    }

    @Override
    public void submit(String docNo, Integer status) {
        if (status != 1) {
            throw new RuntimeException("单据提交异常，请联系管理员！");
        }
        String formSign = getFormSign(docNo);
        switch (formSign) {
            case "QJD":
                oaQjdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaQjdDocument.class).set(OaQjdDocument::getDocStatus, status).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaJbdDocument.class).set(OaJbdDocument::getDocStatus, status).eq(OaJbdDocument::getDocNo, docNo));
        }
        oaDocumentMapper.update(null, Wrappers.lambdaUpdate(OaDocument.class).set(OaDocument::getDocStatus, status).eq(OaDocument::getDocNo, docNo));
    }

    private String getFormSign(String docNo) {
        if (StringUtils.isBlank(docNo)) {
            throw new RuntimeException("单据号不能为空！");
        } else if (docNo.split("-").length != 2 ||
                docNo.split("-")[0].length() > 4 ||
                docNo.split("-")[0].length() < 3 ||
                docNo.split("-")[1].length() != 14) {
            throw new RuntimeException("单据号格式错误！");
        }
        return docNo.split("-")[0];
    }

}
